#include <err.h>
#include <stdio.h>

#include "types.h"
#include "util.h"

char *crit[] = {
	"[!!!!]=",
	"       ",
};

void
battprint(char *buf, size_t len, int acon, int life, int remain, int power)
{
	char c;
	static int frame = 0;

	c = acon ? '=' : ' ';
	if (c == ' ' && remain != 0)
		snprintf(buf, len, " %d%% %d:%02d (%.2g W) |",
		         life, remain/60, remain%60, (float)power/1000000);
	else
		snprintf(buf, len, " %d%% (%.2g W) |", life, (float)power/1000000);

}

#ifdef __OpenBSD__
#include <sys/ioctl.h>

#include <fcntl.h>
#include <unistd.h>

#include <machine/apmvar.h>

int
battread(void *arg, char *buf, size_t len)
{
	struct apm_power_info info;
	int ret, fd;

	fd = open("/dev/apm", O_RDONLY);
	if (fd < 0) {
		warn("open %s", "/dev/apm");
		return -1;
	}
	ret = ioctl(fd, APM_IOC_GETPOWER, &info);
	if (ret < 0) {
		warn("APM_IOC_GETPOWER %s", "/dev/apm");
		close(fd);
		return -1;
	}
	close(fd);

	if (info.battery_state == APM_BATTERY_ABSENT)
		snprintf(buf, len, "");
	else
		battprint(buf, len, info.ac_state == APM_AC_ON,
		          info.battery_life, info.minutes_left);
	return 0;
}
#elif __linux__
int
battread(void *arg, char *buf, size_t len)
{
	FILE *fp;
	int acon, life, energy, power;
	struct battarg *battarg = arg;

	fp = fopen(battarg->cap, "r");
	if (fp == NULL) {
		warn("fopen %s", battarg->cap);
		return -1;
	}
	fscanf(fp, "%d", &life);
	fclose(fp);

	fp = fopen(battarg->ac, "r");
	if (fp == NULL) {
		warn("fopen %s", battarg->ac);
		return -1;
	}
	fscanf(fp, "%d", &acon);
	fclose(fp);
	fp = fopen(battarg->en, "r");
	if (fp == NULL) {
		warn("fopen %s", battarg->en);
		return -1;
	}
	fscanf(fp, "%d", &energy);
	fclose(fp);
	fp = fopen(battarg->pow, "r");
	if (fp == NULL) {
		warn("fopen %s", battarg->pow);
		return -1;
	}
	fscanf(fp, "%d", &power);
	fclose(fp);
	battprint(buf, len, acon, life, (float)energy / power * 60.0, power);
	return 0;
}
#endif
